import logging
import os
import pandas as pd
import matplotlib.pyplot as plt
from collections import Counter

# Set up the logger
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

# Create a file handler
log_file = '/home/dhaarani/Downloads/syslog.log'
file_handler = logging.FileHandler(log_file)
file_handler.setLevel(logging.DEBUG)

# Create a formatter and set it for the handler
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
file_handler.setFormatter(formatter)

# Add the handler to the logger
logger.addHandler(file_handler)

# Log messages at different levels
logger.debug('This is a debug message')
logger.info('This is an info message')
logger.warning('This is a warning message')
logger.error('This is an error message')
logger.critical('This is a critical message')

# Function to check for abnormalities in the log file
def check_abnormalities(log_file):
    abnormalities = []
    with open(log_file, 'r') as file:
        for line in file:
            if 'ERROR' in line or 'CRITICAL' in line:
                abnormalities.append(line.strip())
    return abnormalities

# Function to summarize the log file
def summarize_log_file(log_file):
    log_data = {'DEBUG': 0, 'INFO': 0, 'WARNING': 0, 'ERROR': 0, 'CRITICAL': 0}
    with open(log_file, 'r') as file:
        for line in file:
            if 'DEBUG' in line:
                log_data['DEBUG'] += 1
            elif 'INFO' in line:
                log_data['INFO'] += 1
            elif 'WARNING' in line:
                log_data['WARNING'] += 1
            elif 'ERROR' in line:
                log_data['ERROR'] += 1
            elif 'CRITICAL' in line:
                log_data['CRITICAL'] += 1
    return log_data
    
# Function to find the entity or activity that causes the most abnormalities
def find_most_abnormal_entity(abnormalities):
    entities = [line.split('-')[1].strip() for line in abnormalities]
    entity_counts = Counter(entities)
    most_common_entity = entity_counts.most_common(1)[0]
    return most_common_entity

# Function to find the date or period with the maximum amount of abnormalities based on month
def find_max_abnormalities_period(abnormalities):
    dates = [line.split()[0] for line in abnormalities]
    months = [date[:7] for date in dates]  # Extract year and month (YYYY-MM)
    month_counts = Counter(months)
    max_abnormalities_month = month_counts.most_common(1)[0]
    return max_abnormalities_month

# Function to plot the log data
def plot_log_data(log_data):
    levels = list(log_data.keys())
    counts = list(log_data.values())

    plt.figure(figsize=(10, 6))
    plt.bar(levels, counts, color=['blue', 'green', 'yellow', 'orange', 'red'])
    plt.xlabel('Log Level')
    plt.ylabel('Count')
    plt.title('Log Level Distribution')
    plt.show()


# Check for abnormalities
abnormalities = check_abnormalities(log_file)
abnormality_count = len(abnormalities)
print(f"Total abnormalities found in log file: {abnormality_count}")

# Summarize the log file
log_summary = summarize_log_file(log_file)
print("Log Summary:")
for level, count in log_summary.items():
    print(f"{level}: {count}")

# Find and print the entity or activity that causes the most abnormalities and how many
most_abnormal_entity = find_most_abnormal_entity(abnormalities)
print(f"The entity or activity that causes the most abnormalities is '{most_abnormal_entity[0]}' with {most_abnormal_entity[1]} occurrences.")

# Find and print the date or period with the maximum amount of abnormalities based on month
max_abnormalities_period = find_max_abnormalities_period(abnormalities)
print(f"The period with the maximum amount of abnormalities is in '{max_abnormalities_period[0]}' with {max_abnormalities_period[1]} occurrences.")

# Plot the log data
plot_log_data(log_summary)
